@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2014, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2018, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -80,7 +80,8 @@ class balance_t
            multiplicative<balance_t, long> > > > > > > > > > > > > >
 {
 public:
-  typedef std::map<commodity_t *, amount_t> amounts_map;
+  typedef std::unordered_map<commodity_t *, amount_t> amounts_map;
+  typedef std::vector<const amount_t *> amounts_array;
 
   amounts_map amounts;
 
@@ -185,14 +186,7 @@ public:
    * in time.
    */
   bool operator==(const balance_t& bal) const {
-    amounts_map::const_iterator i, j;
-    for (i = amounts.begin(), j = bal.amounts.begin();
-         i != amounts.end() && j != bal.amounts.end();
-         i++, j++) {
-      if (! (i->first == j->first && i->second == j->second))
-        return false;
-    }
-    return i == amounts.end() && j == bal.amounts.end();
+    return amounts == bal.amounts;
   }
   bool operator==(const amount_t& amt) const {
     if (amt.is_null())
@@ -528,6 +522,12 @@ public:
    */
   balance_t strip_annotations(const keep_details_t& what_to_keep) const;
 
+  /**
+   * Given a balance, insert a commodity-wise sort of the amounts into the
+   * given amounts_array.
+   */
+  void sorted_amounts(amounts_array& sorted) const;
+
   /**
    * Iteration primitives.  `map_sorted_amounts' allows one to visit
    * each amount in balance in the proper order for displaying to the
@@ -594,18 +594,6 @@ public:
       }
     return true;
   }
-
-#if HAVE_BOOST_SERIALIZATION
-private:
-  /** Serialization. */
-
-  friend class boost::serialization::access;
-
-  template<class Archive>
-  void serialize(Archive& ar, const unsigned int /* version */) {
-    ar & amounts;
-  }
-#endif // HAVE_BOOST_SERIALIZATION
 };
 
 inline std::ostream& operator<<(std::ostream& out, const balance_t& bal) {
