@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2016, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2018, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -154,7 +154,7 @@ void truncate_xacts::operator()(post_t& post)
 void sort_posts::post_accumulated_posts()
 {
   std::stable_sort(posts.begin(), posts.end(),
-                   compare_items<post_t>(sort_order));
+                   compare_items<post_t>(sort_order, report));
 
   foreach (post_t * post, posts) {
     post->xdata().drop_flags(POST_EXT_SORT_CALC);
@@ -982,10 +982,12 @@ void interval_posts::flush()
   std::stable_sort(all_posts.begin(), all_posts.end(),
                    sort_posts_by_date());
 
-  // Determine the beginning interval by using the earliest post
-  if (all_posts.size() > 0 && all_posts.front() &&
-      ! interval.find_period(all_posts.front()->date()))
-    throw_(std::logic_error, _("Failed to find period for interval report"));
+  // only if the interval has no start use the earliest post
+  if (!(interval.begin() && interval.find_period(*interval.begin())))
+    // Determine the beginning interval by using the earliest post
+    if (all_posts.size() > 0 && all_posts.front()
+        && !interval.find_period(all_posts.front()->date()))
+      throw_(std::logic_error, _("Failed to find period for interval report"));
 
   // Walk the interval forward reporting all posts within each one
   // before moving on, until we reach the end of all_posts
@@ -1098,17 +1100,17 @@ void posts_as_equity::report_subtotal()
     value_t value(pair.second.value.strip_annotations(report.what_to_keep()));
     if (! value.is_zero()) {
       if (value.is_balance()) {
-        foreach (const balance_t::amounts_map::value_type& amount_pair,
-                 value.as_balance_lval().amounts) {
-          if (! amount_pair.second.is_zero())
-            handle_value(/* value=      */ amount_pair.second,
-                         /* account=    */ pair.second.account,
-                         /* xact=       */ &xact,
-                         /* temps=      */ temps,
-                         /* handler=    */ handler,
-                         /* date=       */ finish,
-                         /* act_date_p= */ false);
-        }
+        value.as_balance_lval().map_sorted_amounts
+          ([&](const amount_t& amt) {
+             if (! amt.is_zero())
+               handle_value(/* value=      */ amt,
+                            /* account=    */ pair.second.account,
+                            /* xact=       */ &xact,
+                            /* temps=      */ temps,
+                            /* handler=    */ handler,
+                            /* date=       */ finish,
+                            /* act_date_p= */ false);
+           });
       } else {
         handle_value(/* value=      */ value.to_amount(),
                      /* account=    */ pair.second.account,
@@ -1243,19 +1245,34 @@ void generate_posts::add_post(const date_interval_t& period, post_t& post)
 
 void budget_posts::report_budget_items(const date_t& date)
 {
+  { // Cleanup pending items that finished before date
+    // We have to keep them until the last day they apply because operator() needs them to see if a
+    // posting is budgeted or not
+    std::list<pending_posts_list::iterator> posts_to_erase;
+    for (pending_posts_list::iterator i = pending_posts.begin(); i != pending_posts.end(); i++) {
+      pending_posts_list::value_type& pair(*i);
+      if (pair.first.finish && ! pair.first.start && pair.first.finish < date) {
+        posts_to_erase.push_back(i);
+      }
+    }
+    foreach (pending_posts_list::iterator& i, posts_to_erase)
+      pending_posts.erase(i);
+  }
+  
   if (pending_posts.size() == 0)
     return;
 
   bool reported;
   do {
-    std::list<pending_posts_list::iterator> posts_to_erase;
-
     reported = false;
     for (pending_posts_list::iterator i = pending_posts.begin();
          i != pending_posts.end();
          i++) {
       pending_posts_list::value_type& pair(*i);
 
+      if (pair.first.finish && ! pair.first.start)
+        continue;               // skip expired posts
+
       optional<date_t> begin = pair.first.start;
       if (! begin) {
         optional<date_t> range_begin;
@@ -1283,9 +1300,6 @@ void budget_posts::report_budget_items(const date_t& date)
         post_t& post = *pair.second;
 
         ++pair.first;
-        if (! pair.first.start)
-          posts_to_erase.push_back(i);
-
         DEBUG("budget.generate", "Reporting budget for "
               << post.reported_account()->fullname());
 
@@ -1310,9 +1324,6 @@ void budget_posts::report_budget_items(const date_t& date)
         reported = true;
       }
     }
-
-    foreach (pending_posts_list::iterator& i, posts_to_erase)
-      pending_posts.erase(i);
   } while (reported);
 }
 
