@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2014, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2019, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -58,7 +58,7 @@ namespace {
    */
   int do_fork(std::ostream ** os, const path& pager_path)
   {
-#ifndef WIN32
+#if !defined(_WIN32) && !defined(__CYGWIN__)
     int pfd[2];
 
     int status = pipe(pfd);
@@ -89,11 +89,7 @@ namespace {
     else {                      // parent
       close(pfd[0]);
       typedef iostreams::stream<iostreams::file_descriptor_sink> fdstream;
-#if BOOST_VERSION >= 104400
       *os = new fdstream(pfd[1], iostreams::never_close_handle);
-#else // BOOST_VERSION >= 104400
-      *os = new fdstream(pfd[1]);
-#endif // BOOST_VERSION >= 104400
     }
     return pfd[1];
 #else
@@ -115,7 +111,7 @@ void output_stream_t::initialize(const optional<path>& output_file,
 
 void output_stream_t::close()
 {
-#ifndef WIN32
+#if !defined(_WIN32) && !defined(__CYGWIN__)
   if (os != &std::cout) {
     checked_delete(os);
     os = &std::cout;
