@@ -12,8 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#if !(defined(_WIN32) || defined(__CYGWIN__))
+#error This file should only be compiled on Windows.
+#endif
 
-#ifdef WIN32
 // Implement strptime under windows
 
 #include "strptime.h"
@@ -22,6 +24,22 @@
 #include <ctype.h>
 #include <string.h>
 
+#if defined(__CYGWIN__)
+// Define strnicmp for Cygwin.
+#ifndef strcmpi
+#define strcmpi strcasecmp
+#endif
+#ifndef stricmp
+#define stricmp strcasecmp
+#endif
+#ifndef strncmpi
+#define strncmpi strncasecmp
+#endif
+#ifndef strnicmp
+#define strnicmp strncasecmp
+#endif
+#endif
+
 static const char* kWeekFull[] = {
   "Sunday", "Monday", "Tuesday", "Wednesday",
   "Thursday", "Friday", "Saturday"
@@ -70,14 +88,14 @@ static char* _strptime(const char *s, const char *format, struct tm *tm) {
       case 'A':
         tm->tm_wday = -1;
         for (int i = 0; i < 7; ++i) {
-          len = static_cast<int>(strlen(kWeekAbbr[i]));
-          if (strnicmp(kWeekAbbr[i], s, len) == 0) {
+          len = static_cast<int>(strlen(kWeekFull[i]));
+          if (strnicmp(kWeekFull[i], s, len) == 0) {
             tm->tm_wday = i;
             break;
           }
 
-          len = static_cast<int>(strlen(kWeekFull[i]));
-          if (strnicmp(kWeekFull[i], s, len) == 0) {
+          len = static_cast<int>(strlen(kWeekAbbr[i]));
+          if (strnicmp(kWeekAbbr[i], s, len) == 0) {
             tm->tm_wday = i;
             break;
           }
@@ -92,14 +110,14 @@ static char* _strptime(const char *s, const char *format, struct tm *tm) {
       case 'h':
         tm->tm_mon = -1;
         for (int i = 0; i < 12; ++i) {
-          len = static_cast<int>(strlen(kMonthAbbr[i]));
-          if (strnicmp(kMonthAbbr[i], s, len) == 0) {
+          len = static_cast<int>(strlen(kMonthFull[i]));
+          if (strnicmp(kMonthFull[i], s, len) == 0) {
             tm->tm_mon = i;
             break;
           }
 
-          len = static_cast<int>(strlen(kMonthFull[i]));
-          if (strnicmp(kMonthFull[i], s, len) == 0) {
+          len = static_cast<int>(strlen(kMonthAbbr[i]));
+          if (strnicmp(kMonthAbbr[i], s, len) == 0) {
             tm->tm_mon = i;
             break;
           }
@@ -185,5 +203,3 @@ static char* _strptime(const char *s, const char *format, struct tm *tm) {
 char* strptime(const char *buf, const char *fmt, struct tm *tm) {
   return _strptime(buf, fmt, tm);
 }
-
-#endif  // WIN32
