@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2014, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2019, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -129,11 +129,7 @@ value_t::operator bool() const
 void value_t::set_type(type_t new_type)
 {
   if (new_type == VOID) {
-#if BOOST_VERSION >= 103700
     storage.reset();
-#else
-    storage = intrusive_ptr<storage_t>();
-#endif
   } else {
     if (! storage || storage->refc > 1)
       storage = new storage_t;
@@ -341,6 +337,10 @@ value_t& value_t::operator+=(const value_t& val)
   }
 
   switch (type()) {
+  case VOID:
+    *this = value_t(val);
+    return *this;
+
   case DATETIME:
     switch (val.type()) {
     case INTEGER:
@@ -887,7 +887,7 @@ bool value_t::is_less_than(const value_t& val) const
           ! val.as_amount().has_commodity())
         return as_amount() < val.as_amount();
       else
-        return commodity_t::compare_by_commodity()(&as_amount(), &val.as_amount());
+        return commodity_t::compare_by_commodity()(&as_amount(), &val.as_amount()) < 0;
     case BALANCE:
       return val.to_amount() > as_amount();
     default:
